import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Attribute;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

/**
 * A simple program that automatically generates implementations 
 * for the feature model based on the templates and the model.xml file.
 * Parses the model file using jsoup and creates a Main Configurator
 * and a .jak for each module.
 *  
 */


public class ImplGenerator {
	public static void main(String[] aArgs) {
		ImplGenerator oGenerator = new ImplGenerator();
	}
	
	//input path for the model file
	String m_sFeatureModelPath = "../FeatureModelConfigurator/model.xml";
	//input path for the main configurator template
	String m_sMainTemplatePath = "./template/Main.jak";
	//input path for the module template
	String m_sModuleTemplatePath = "./template/Module.jak";
	
	private ImplGenerator() {
		//create jsoup document from model file
		Document oFeatureModelDoc;
		try {
			File oModel = new File(m_sFeatureModelPath);
			oFeatureModelDoc = Jsoup.parse(oModel, "UTF-8");
			//System.out.println(oFeatureModelDoc.toString());
		}
		catch (IOException oEx) {
			System.out.print(oEx);
			return;
		}
		//create main configurator file
		String sDestPath = "../FeatureModelConfigurator/features/ConfiguratorMain/Configurator.jak";
		CopyFile(m_sMainTemplatePath, sDestPath);
		
		//get modules
		Elements aModules = oFeatureModelDoc.getElementsByAttributeValue("name", "ConfiguratorMain").first().children();
		FindNonAbstractModules(aModules);		
	}
	
	//recusively iterate through modules, create files for all non-abstract modules
	private void FindNonAbstractModules(Elements aModules) {
		for(Element oModule : aModules) {
			//if the Element is not a graphics node
			if(oModule.tagName().equals("graphics")) {
				continue;
			}
			//if the module is not abstract
			List<Attribute> aAttributes = oModule.attributes().asList();
			if(!aAttributes.contains(new Attribute("abstract", "true"))) {
				String sModuleName = oModule.attr("name");
				if(sModuleName==null) {
					System.out.println("Couldn't get Module name");
					return;
				}
				String sTargetPath = "../FeatureModelConfigurator/features/"
						+ sModuleName + "/Configurator.jak";
				//copy the file
				CopyFile(m_sModuleTemplatePath, sTargetPath);
				//edit the file and add the name
				Path path = Paths.get(sTargetPath);				
				try {
					//write to file
					String sContent = new String(Files.readAllBytes(path), StandardCharsets.UTF_8);
					sContent = sContent.replaceAll("ModuleName", sModuleName);
					Files.write(path, sContent.getBytes(StandardCharsets.UTF_8));
				}
				catch (IOException oEx) {
					System.out.print(oEx.toString());
				}
			}
			//if the module has children, iterate through them
			if(oModule.children()!=null) {
				FindNonAbstractModules(oModule.children());
			}
		}
	}
	
	//copy a file from source to dest
	private void CopyFile(String sSource, String sDest) {
		File oSourceFile = new File(sSource);
		File oDestFile = new File(sDest);
		
		try (FileInputStream fis = new FileInputStream(oSourceFile);
				FileOutputStream fos = new FileOutputStream(oDestFile)) {

	            byte[] aBuffer = new byte[1024];
	            int nLength;

	            while ((nLength = fis.read(aBuffer)) > 0) {
	                fos.write(aBuffer, 0, nLength);
	            }
	            
	            System.out.println("copied " + sSource + " to " + sDest);
	    }
		catch (IOException oEx) {
			System.out.print(oEx);
		}
	}
}
